"""update oauth2_code table

Revision ID: b30c7818c902
Revises: 67ae8866b18f
Create Date: 2025-09-27 13:21:54.189272

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'b30c7818c902'
down_revision = '67ae8866b18f'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table('oauth2_code', schema=None) as batch_op:
        batch_op.add_column(sa.Column('acr', sa.Text(), nullable=True))
        batch_op.add_column(sa.Column('amr', sa.Text(), nullable=True))

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table('oauth2_code', schema=None) as batch_op:
        batch_op.drop_column('amr')
        batch_op.drop_column('acr')

    # ### end Alembic commands ###
