"""Add 'notes' column to 'Activity' table

Revision ID: 096dd0b43beb
Revises: 71093ac9ca44
Create Date: 2018-06-13 17:56:20.359884

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '096dd0b43beb'
down_revision = '71093ac9ca44'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('activities', sa.Column('notes', sa.String(length=500), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('activities', 'notes')
    # ### end Alembic commands ###
